﻿Public Structure Pitch
    Private Shared ReadOnly semitone As Double = Math.Pow(2, 1.0 / 12)

    Private Shared ReadOnly middleC As Double = 440 * Math.Pow(semitone, 3) / 2

    ' Middle C is 64
    Public Sub New(ByVal midiNumber As Integer)
        Me.New(CType(midiNumber Mod 12, Note), midiNumber \ 12 - 1)
    End Sub
    
    Public Sub New(ByVal note As Note, ByVal octave As Integer)
        Me.Note = note
        Me.Octave = octave
        Me.MidiNumber = 12 * (Me.Octave + 1) + CInt(Me.Note)
        Me.Frequency = middleC * Math.Pow(semitone, CInt(note)) * Math.Pow(2, Me.Octave - 4)
    End Sub

    Public Property Note As Note

    Public Property Octave As Integer

    Public Property MidiNumber As Integer

    Public Property Frequency As Double

    Public ReadOnly Property IsNatural As Boolean
        Get
            Return Note = Note.C OrElse Note = Note.D OrElse Note = Note.E OrElse Note = Note.F OrElse Note = Note.G OrElse Note = Note.A OrElse Note = Note.B
        End Get
    End Property

    Public Overrides Function ToString() As String
        Dim note As String = Me.Note.ToString

        Return String.Format("{0}{1}{2} {3} ({4}) {5}Hz", note.Chars(0), If(note.EndsWith("sharp"), ChrW(&H266F).ToString(), ""), If(note.EndsWith("flat"), ChrW(&H266D).ToString(), ""), Me.Octave, Me.MidiNumber, Me.Frequency)
    End Function
End Structure
