﻿Imports System.Windows.Navigation
Imports Petzold.MusicSynthesis

Partial Public Class MainPage
    Inherits PhoneApplicationPage

    Private Const SAMPLE_RATE As Integer = 22050

    Private Shared ReadOnly MIN_PITCH As New Pitch(Note.C, 3)

    Private Shared ReadOnly MAX_PITCH As New Pitch(Note.C, 7)

    Private Shared ReadOnly MIN_FREQ As Double = MIN_PITCH.Frequency

    Private Shared ReadOnly MAX_FREQ As Double = MAX_PITCH.Frequency

    Private Shared ReadOnly MIN_FREQ_LOG2 As Double = Math.Log(MIN_FREQ) / Math.Log(2)

    Private Shared ReadOnly MAX_FREQ_LOG2 As Double = Math.Log(MAX_FREQ) / Math.Log(2)

    Private mixer As Mixer

    Private oscillator As ThereminOscillator

    Private xStart As Double ' the X coordinate corresponding to MIN_PITCH

    Private xDelta As Integer ' the number of pixels per semitone

    Private touchID As Integer = Int32.MinValue

    Public Sub New()
        InitializeComponent()

        ' Create Oscillator, Mixer and MixerInput objects
        oscillator = New ThereminOscillator(SAMPLE_RATE)
        mixer = New Mixer
        mixer.Inputs.Add(New StereoToMonoMixerInput(oscillator))

        ' Install event handlers
    End Sub

    Private Sub OnLoaded(ByVal sender As Object, ByVal args As EventArgs) Handles Me.Loaded
        Dim count As Integer = MAX_PITCH.MidiNumber - MIN_PITCH.MidiNumber
        xDelta = CInt((ContentPanel.ActualWidth - 4) / count)
        xStart = CInt((ContentPanel.ActualWidth - count * xDelta) / 2)

        Dim cBrush As Brush = New SolidColorBrush(Colors.Red) ' C
        Dim fBrush As Brush = New SolidColorBrush(Colors.Blue) ' F
        Dim nBrush As Brush = TryCast(Me.Resources("PhoneForegroundBrush"), Brush) ' other naturals
        Dim aBrush As Brush = New SolidColorBrush(Colors.Gray) ' accidentals
        Dim x As Double = xStart + 0.5

        For i = MIN_PITCH.MidiNumber To MAX_PITCH.MidiNumber
            Dim pitch As New Pitch(i)

            Dim line As Line = New Line With {.X1 = x, .Y1 = 0, .X2 = x, .Y2 = ContentPanel.ActualHeight, .Stroke = If(pitch.Note = Note.C, cBrush, (If(pitch.Note = Note.F, fBrush, (If(pitch.IsNatural, nBrush, aBrush)))))}

            ContentPanel.Children.Add(line)

            x += xDelta
        Next i
    End Sub

    Protected Overrides Sub OnNavigatedTo(ByVal args As NavigationEventArgs)
        Dim dynamicPcmStreamSource As New DynamicPcmStreamSource(SAMPLE_RATE, 15)
        dynamicPcmStreamSource.SampleProvider = mixer
        mediaElement.SetSource(dynamicPcmStreamSource)

        MyBase.OnNavigatedTo(args)
    End Sub

    WithEvents TouchEvent As Touch
    Private Sub OnTouchFrameReported(ByVal sender As Object, ByVal args As TouchFrameEventArgs) Handles TouchEvent.FrameReported
        Dim touchPoints As TouchPointCollection = args.GetTouchPoints(ContentPanel)

        For Each touchPoint In touchPoints
            Dim pt As Point = touchPoint.Position
            Dim id As Integer = touchPoint.TouchDevice.Id

            Select Case touchPoint.Action
                Case TouchAction.Down
                    oscillator.Amplitude = ThereminOscillator.MIN_AMPLITUDE
                    oscillator.DestinationAmplitude = CalculateAmplitude(pt.Y)
                    oscillator.Frequency = CalculateFrequency(pt.X)
                    oscillator.DestinationFrequency = oscillator.Frequency
                    HighlightLines(pt.X, True)
                    touchID = id

                Case TouchAction.Move
                    If id = touchID Then
                        oscillator.DestinationFrequency = CalculateFrequency(pt.X)
                        oscillator.DestinationAmplitude = CalculateAmplitude(pt.Y)
                        HighlightLines(pt.X, True)
                    End If

                Case TouchAction.Up
                    If id = touchID Then
                        oscillator.DestinationAmplitude = 0
                        touchID = Int32.MinValue

                        ' Remove highlighting
                        HighlightLines(0, False)
                    End If
            End Select
        Next touchPoint
    End Sub

    Private Function CalculateAmplitude(ByVal y As Double) As Double
        Return Math.Min(1, Math.Pow(10, -4 * (1 - y / ContentPanel.ActualHeight)))
    End Function

    Private Function CalculateFrequency(ByVal x As Double) As Double
        Return Math.Pow(2, MIN_FREQ_LOG2 + (x - xStart) / xDelta \ 12)
    End Function

    Private Sub HighlightLines(ByVal x As Double, ByVal highlight As Boolean)
        For Each child In ContentPanel.Children
            If TypeOf child Is Line Then
                Dim line As Line = TryCast(child, Line)
                line.StrokeThickness = If(highlight, Math.Max(1, xDelta - Math.Abs(x - line.X1)), 1)
            End If
        Next child
    End Sub
End Class
